#!/bin/bash

HOSTAPD_CONF="/etc/hostapd/hostapd.conf"
INTERFACES="/etc/network/interfaces"
DNSMASQ_CONF="/etc/dnsmasq.conf"
DHCPCD_CONF="/etc/dhcpcd.conf"
WPA_CONF="/etc/wpa_supplicant/wpa_supplicant.conf"
OS_ID=$(cat /etc/os-release | sed -n 's/^ID=\(.*\)/\1/p')
OS_VERSION=$(cat /etc/os-release | sed -n 's/^VERSION="[0-9]\s(\(.*\))"/\1/p')
MAINUSER=$(getent passwd 1000 | cut -d: -f1)
DECONZ_CONF_DIR="/home/$MAINUSER/.local/share"
DECONZ_PORT=80
WIFI_DEV=wlan0
ZLLDB=""
TIMEOUT=0      # main loop iteration timeout, can be controlled by SIGUSR1
OWN_PID=$$

AP_NAME=""     # empty string | Not set | value
AP_PW=""       # empty string | value
WIFI_NAME=""   # empty string | Not set | value
WIFI_PW=""     # empty string | value
ERROR_COUNT=0  # counts "connect to wifi" fails
IP_ADDR=""     # ip addr if connection was successfull
MAX_ERRORS=1

LOG_LEVEL=6

LOG_EMERG=
LOG_ALERT=
LOG_CRIT=
LOG_ERROR=
LOG_WARN=
LOG_NOTICE=
LOG_INFO=
LOG_DEBUG=

[[ $LOG_LEVEL -ge 0 ]] && LOG_EMERG="<0>"
[[ $LOG_LEVEL -ge 1 ]] && LOG_ALERT="<1>"
[[ $LOG_LEVEL -ge 2 ]] && LOG_CRIT="<2>"
[[ $LOG_LEVEL -ge 3 ]] && LOG_ERROR="<3>"
[[ $LOG_LEVEL -ge 4 ]] && LOG_WARN="<4>"
[[ $LOG_LEVEL -ge 5 ]] && LOG_NOTICE="<5>"
[[ $LOG_LEVEL -ge 6 ]] && LOG_INFO="<6>"
[[ $LOG_LEVEL -ge 7 ]] && LOG_DEBUG="<7>"

# format of sqlite3 query result:
# <key>|<value>

OLD_AP_NAME=""
OLD_AP_PW=""
OLD_WIFI_NAME=""
OLD_WIFI_PW=""
RESTORE_AP=false
WIFI_CONFIG=""

function putWifiUpdated() {
	curl --noproxy '*' -s -o /dev/null -d "$@" -X PUT http://127.0.0.1:${DECONZ_PORT}/api/$OWN_PID/config/wifi/updated
}

function init {
    # is sqlite3 installed?
	sqlite3 --version &> /dev/null
	if [ $? -ne 0 ]; then
		[[ $LOG_WARN ]] && echo "${LOG_WARN}sqlite3 not installed"
		return
	fi

	# look for latest config in specific order
	drs=("data/dresden-elektronik/deCONZ/zll.db" "dresden-elektronik/deCONZ/zll.db" "deCONZ/zll.db")
	for i in "${drs[@]}"; do
		if [ -f "${DECONZ_CONF_DIR}/$i" ]; then
			ZLLDB="${DECONZ_CONF_DIR}/$i"
			break
		fi
	done

	if [ ! -f "$ZLLDB" ]; then
		# might have been deleted
		ZLLDB=""
	fi

	if [[ -z "$ZLLDB" ]]; then
		[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}no database file (zll.db) found"
		return
	fi

	# get deCONZ REST-API port
	local value=$(sqlite3 $ZLLDB "select * from config2 where key=\"port\"")
	if [ $? -ne 0 ]; then
		[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}no deCONZ port found in database"
		return
	fi

	value=$(echo $value | cut -d'|' -f2)
	if [[ -n $value ]]; then
		DECONZ_PORT=$value
		[[ $LOG_INFO ]] && echo "${LOG_INFO}use deCONZ port $DECONZ_PORT"
	fi

	# get wifi interface
	for dev in `ls /sys/class/net`; do
		if [ -d "/sys/class/net/$dev/wireless" ]; then
			WIFI_DEV=$dev
			[[ $LOG_INFO ]] && echo "${LOG_INFO}use wifi interface $WIFI_DEV"
		fi
	done
}

#trap 'on_usr2;' SIGUSR2
#on_usr2() {
#}

function checkWifiConfig {

	# mgmt flags:
	#          0x01  hostapd
	#          0x02  wpa_supplicant
	#          0x04  interfaces
	#          0x08  active
	local mgmt_hostapd=0x01
	local mgmt_wpa_supplicant=0x02
	local mgmt_interfaces=0x04
	local mgmt_active=0x08
	local mgmt=0
	local CONFIGURED_BY_DECONZ=false
	SSID_AP=""
	SSID_CLIENT=""

	# check current state and which files are managed by deCONZ

	[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}forward current configuration"

	[[ -e "$HOSTAPD_CONF"  && -n $(grep "by deCONZ" $HOSTAPD_CONF) ]] && mgmt=$((mgmt | mgmt_hostapd))
	[[ -e "$WPA_CONF" && -n $(grep "by deCONZ" $WPA_CONF) ]] && mgmt=$((mgmt | mgmt_wpa_supplicant))
	[[ -e "$INTERFACES" && -n $(grep "by deCONZ" $INTERFACES) ]] && mgmt=$((mgmt | mgmt_interfaces))

	[[ $mgmt -ne 0 ]] && CONFIGURED_BY_DECONZ=true

	USER_SSID=$(iw $WIFI_DEV 2> /dev/null info | grep ssid | sed -e 's/\<ssid\>//g') #active SSID
	if [[ -n $USER_SSID ]]; then #  get active configuration
		USER_SSID=$(echo $USER_SSID | xargs) # trim whitespace
		mgmt=$((mgmt | mgmt_active)) # set active flag
		iw $WIFI_DEV info 2> /dev/null | grep 'type AP' > /dev/null
		if [ $? -eq 0 ]; then
			SSID_AP=$USER_SSID
		fi

		iw $WIFI_DEV info  2> /dev/null | grep 'type managed' > /dev/null
		if [ $? -eq 0 ]; then
			SSID_CLIENT=$USER_SSID
		fi
	else
		# configured but not  active?
		if [ -f "$HOSTAPD_CONF" ]; then
			USER_SSID=$(cat $HOSTAPD_CONF | grep ssid= -m1 | cut -d'=' -f2)
			if [[ -n $USER_SSID ]]; then
				SSID_AP=$USER_SSID
			fi
		fi

		if [ -f "$WPA_CONF" ]; then
			USER_SSID=$(cat $WPA_CONF | grep ssid= -m1 | cut -d'"' -f2)
			if [[ -n $USER_SSID ]]; then
				SSID_CLIENT=$USER_SSID
			fi
		fi
	fi

	if [[ -n $SSID_AP ]]; then
		if [[ $((mgmt & mgmt_hostapd)) -eq 0 ]]; then
			CONFIGURED_BY_DECONZ=false
		fi
		putWifiUpdated "{\"status\":\"current-config\", \"type\":\"accesspoint\", \"ssid\":\"$SSID_AP\", \"mgmt\": $mgmt }"
	fi

	if [[ -n $SSID_CLIENT ]]; then
		if [[ $((mgmt & mgmt_wpa_supplicant)) -eq 0 ]]; then
			CONFIGURED_BY_DECONZ=false
		fi
		putWifiUpdated "{\"status\":\"current-config\", \"type\":\"client\", \"ssid\":\"$SSID_CLIENT\", \"mgmt\": $mgmt}"
	fi

	if [[ -z $USER_SSID ]]; then
		putWifiUpdated "{\"status\":\"current-config\", \"mgmt\":\"$mgmt\"}"
	fi

	# get database config

	params=( [0]="wifi" [1]="wifitype" [2]="wifiname" [3]="wifiappw" [4]="wificlientname" [5]="wificlientpw" )
	values=()

	for i in {0..1}; do
		param=${params[$i]}
		value=$(sqlite3 $ZLLDB "select * from config2 where key=\"${param}\"")
		if [ $? -ne 0 ]; then
			return
		fi

		value=$(echo $value | cut -d'|' -f2)

		# basic check for non empty; not set
		if [[ ! -z "$value" && "$value" != "Not set" ]]; then
			values[$i]=$(echo $value | cut -d'|' -f2)
		fi
	done

	# all parameters found and valid?
	if [ -z "${values[0]}" ]; then
			[[ $LOG_ERROR ]] && echo "${LOG_ERROR}missing parameter wifi"
			return
		fi
		if [ -z "${values[1]}" ]; then
			[[ $LOG_ERROR ]] && echo "${LOG_ERROR}missing parameter wifitype"
			return
		fi

	local wifiConfig="${values[0]}"
	local wifiType="${values[1]}"

	WIFI_CONFIG="${values[0]}"

	if [ "$wifiConfig" = "not-available" ]; then
		return
	fi

	if [ "$wifiConfig" = "not-configured" ]; then
		[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}wifi not configured by deCONZ - cancel"
		return
	fi

	if [[ "$wifiConfig" = "deactivated" ]]; then

		[[ $CONFIGURED_BY_DECONZ = false ]] && return

		# deactivate wifi
		[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}wifi deactivated"
		systemctl -q is-active hostapd
		if [ $? -eq 0 ]; then
			echo "${LOG_INFO}hostapd is running - deactivate it"
			systemctl stop hostapd
		fi
		systemctl -q is-active wpa_supplicant
		if [ $? -eq 0 ]; then
			[[ $LOG_INFO ]] && echo "${LOG_INFO}wpa_supplicant is running - deactivate it"
			[[ $LOG_INFO ]] && echo "${LOG_INFO}release any dhcp leases"
			systemctl stop wpa_supplicant
			pkill wpa_supplicant
		fi

		return
	fi

	if [[ $CONFIGURED_BY_DECONZ = false ]]; then

	    [[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}wifi not configured by deCONZ"

		if [[ "$wifiConfig" != "new-configured" ]]; then
			return
		fi

		[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}create new wifi config from app"
	fi

	# client, accesspoint and accesspoint-backup
	if [[ "$wifiType" != *"accesspoint"* ]] && [[ "$wifiType" != "client" ]]; then
		return
	fi

	### accesspoint/accesspoint-backup mode
	if [[ "$wifiType" = *"accesspoint"* ]]; then
		[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}enter configure ap section"
		for i in {2..3}; do
			param=${params[$i]}
			value=$(sqlite3 $ZLLDB "select * from config2 where key=\"${param}\"")
			if [ $? -ne 0 ]; then
				return
			fi

			value=$(echo $value | cut -d'|' -f2)

			# basic check for non empty; not set
			if [[ ! -z "$value" && "$value" != "Not set" ]]; then
				values[$i]=$(echo $value | cut -d'|' -f2)
			fi
		done

		# all parameters found and valid?
		if [ -z "${values[2]}" ]; then
			[[ $LOG_ERROR ]] && echo "${LOG_ERROR}missing parameter wifiname"
			return
		fi
		if [ -z "${values[3]}" ]; then
			[[ $LOG_ERROR ]] && echo "${LOG_ERROR}missing parameter wifiappw"
			return
		fi

		which hostapd &> /dev/null
		if [ $? -ne 0 ]; then
			[[ $LOG_CRIT ]] && echo "${LOG_CRIT}hostapd not installed"
			return
		fi

		which dnsmasq &> /dev/null
		if [ $? -ne 0 ]; then
			[[ $LOG_CRIT ]] && echo "${LOG_CRIT}dnsmasqd not installed"
			return
		fi

		AP_NAME=${values[2]}
		AP_PW=${values[3]}

		if [ -f "$HOSTAPD_CONF" ]; then
			# check from config
			OLD_AP_NAME=$(cat $HOSTAPD_CONF | grep "^ssid" | cut -d'=' -f2)
			OLD_AP_PW=$(cat $HOSTAPD_CONF | grep "^wpa_passphrase" | cut -d'=' -f2)
		fi

		# restart hostapd if hostapd not running
		pidof hostapd > /dev/null
		if [[ $? -ne 0 ]]; then
			[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}hostapd not running"
		fi

		iw $WIFI_DEV info 2> /dev/null | grep 'type AP' > /dev/null
		if [[ $? -ne 0 ]] || [[ -z $IP_ADDR ]]; then
			[[ $LOG_WARN ]] && echo "${LOG_WARN}accesspoint not running"
			RESTORE_AP=true
		fi

		# reconfigure if parameter changed or services aren't started or accesspoint must be restored
		if [[ $RESTORE_AP = true ]]; then
		    [[ $LOG_WARN ]] && echo "${LOG_WARN}restoring ap"
		    RESTORE_AP=false
		    # continue
		else
			if [[ $AP_NAME == $OLD_AP_NAME ]] && [[ $AP_PW == $OLD_AP_PW ]]; then

				[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}config is up to date"
				systemctl -q is-active hostapd
				if [ $? -eq 0 ]; then
					[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}hostapd is running"
					systemctl -q is-active dnsmasq
					if [ $? -eq 0 ]; then
						[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}dnsmasq is running"
						putWifiUpdated "{\"status\":\"ap-alive\",\"type\":\"$wifiType\"}"
						if [[ "$wifiConfig" != "configured" ]]; then
							sqlite3 $ZLLDB "replace into config2 (key, value) values('wifi', 'configured')" &> /dev/null
						fi
						return
					else
						[[ $LOG_WARN ]] && echo "${LOG_WARN}dnsmasq is not running - reconfigure"
					fi
				else
					[[ $LOG_WARN ]] && echo "${LOG_WARN}hostapd is not running - reconfigure"
				fi
			else
				[[ $LOG_INFO ]] && echo "${LOG_INFO}ap config changed - reconfigure"
			fi # reconfigure if parameter changed
		fi # restore ap

		## stop network
		[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}stopping network"
		putWifiUpdated '{"status":"ap-connecting"}'

		systemctl stop dhcpcd
		systemctl stop dnsmasq
		systemctl stop hostapd
		systemctl stop wpa_supplicant
		pkill wpa_supplicant

		# remove all ip addresses
		ip addr flush dev $WIFI_DEV

		[[ $LOG_INFO ]] && echo "${LOG_INFO}configuring AP"

		## create hostapd conf
		echo "# generated by deCONZ
interface=$WIFI_DEV
#driver=nl80211

# AP configuration
ssid=$AP_NAME
channel=11
hw_mode=g
wmm_enabled=1
#country_code=DE
#ieee80211d=1
ignore_broadcast_ssid=0
auth_algs=1

# AP security
wpa=2
wpa_key_mgmt=WPA-PSK
rsn_pairwise=CCMP
wpa_passphrase=$AP_PW" > $HOSTAPD_CONF

		## Add conf to hostapd daemon
		sed -i '/DAEMON_CONF=/c\DAEMON_CONF=\"\/etc\/hostapd\/hostapd.conf\"' /etc/default/hostapd

		## Static address for access point
			echo "# generated by deCONZ
# A sample configuration for dhcpcd.
# See dhcpcd.conf(5) for details.

# Allow users of this group to interact with dhcpcd via the control socket.
#controlgroup wheel

# Inform the DHCP server of our hostname for DDNS.
hostname

# Use the hardware address of the interface for the Client ID.
clientid
# or
# Use the same DUID + IAID as set in DHCPv6 for DHCPv4 ClientID as per RFC4361.
# Some non-RFC compliant DHCP servers do not reply with this set.
# In this case, comment out duid and enable clientid above.
#duid

# Persist interface configuration when dhcpcd exits.
persistent

# Rapid commit support.
# Safe to enable by default because it requires the equivalent option set
# on the server to actually work.
option rapid_commit

# A list of options to request from the DHCP server.
option domain_name_servers, domain_name, domain_search, host_name
option classless_static_routes
# Most distributions have NTP support.
option ntp_servers
# Respect the network MTU. This is applied to DHCP routes.
option interface_mtu

# A ServerID is required by RFC2131.
require dhcp_server_identifier

# Generate Stable Private IPv6 Addresses instead of hardware based ones
slaac private

interface $WIFI_DEV
static ip_address=192.168.8.1/24" > $DHCPCD_CONF



# --------------- default for Raspbian Jessi --- /etc/network/interfaces
# interfaces(5) file used by ifup(8) and ifdown(8)

# Please note that this file is written to be used with dhcpcd
# For static IP, consult /etc/dhcpcd.conf and 'man dhcpcd.conf'

# Include files from /etc/network/interfaces.d:
#source-directory /etc/network/interfaces.d

#auto lo
#iface lo inet loopback

#iface eth0 inet manual

#allow-hotplug wlan0
#iface wlan0 inet manual
#    wpa-conf /etc/wpa_supplicant/wpa_supplicant.conf

#allow-hotplug wlan1
#iface wlan1 inet manual
#    wpa-conf /etc/wpa_supplicant/wpa_supplicant.conf
#----------------------------------------------------------



# --------------- default for Raspbian Stretch --- /etc/network/interfaces
# interfaces(5) file used by ifup(8) and ifdown(8)

# Please note that this file is written to be used with dhcpcd
# For static IP, consult /etc/dhcpcd.conf and 'man dhcpcd.conf'

# Include files from /etc/network/interfaces.d:
#source-directory /etc/network/interfaces.d
#----------------------------------------------------------


if [ "$OS_ID" = "raspbian" ] && [ "$OS_VERSION" = "jessie" ]; then

		echo "# generated by deCONZ
# interfaces(5) file used by ifup(8) and ifdown(8)

# Please note that this file is written to be used with dhcpcd
# For static IP, consult /etc/dhcpcd.conf and 'man dhcpcd.conf'

# Include files from /etc/network/interfaces.d:
source-directory /etc/network/interfaces.d

auto lo
iface lo inet loopback

iface eth0 inet manual

allow-hotplug $WIFI_DEV
iface $WIFI_DEV inet manual
    wpa-conf /etc/wpa_supplicant/wpa_supplicant.conf

allow-hotplug wlan1
iface wlan1 inet manual
    wpa-conf /etc/wpa_supplicant/wpa_supplicant.conf" > $INTERFACES
fi


# stretch uses dhcpcd hooks for wpa_supplicant
if [ "$OS_ID" = "raspbian" ] && [ "$OS_VERSION" = "stretch" ]; then
			echo "# generated by deCONZ
# interfaces(5) file used by ifup(8) and ifdown(8)

# Please note that this file is written to be used with dhcpcd
# For static IP, consult /etc/dhcpcd.conf and 'man dhcpcd.conf'

# Include files from /etc/network/interfaces.d:
source-directory /etc/network/interfaces.d" > $INTERFACES
fi

		## Configure dns server
		echo "# generated by deCONZ
interface=$WIFI_DEV
no-dhcp-interface=lo,eth0
dhcp-range=192.168.8.50,192.168.8.150,24h
dhcp-option=option:dns-server,192.168.8.1
address=/phoscon.app/192.168.8.1
address=/phoscon.lan/192.168.8.1" > $DNSMASQ_CONF

		# empty wpa_supplicant file
		echo "# generated by deCONZ" > $WPA_CONF

		## restart network
		[[ $LOG_INFO ]] && echo "${LOG_INFO}restarting network"

		systemctl restart networking
		systemctl daemon-reload
		systemctl restart dhcpcd
		systemctl restart dnsmasq
		systemctl restart hostapd


		for i in {1..10}; do
			sleep 2
			checkIpAddr

			iw $WIFI_DEV info 2> /dev/null | grep 'type AP' > /dev/null
			if [[ $? -eq 0 ]] || [[ -n $IP_ADDR ]]; then
				[[ $LOG_INFO ]] && echo "${LOG_INFO}Accesspoint created"

				if [[ "$wifiConfig" != "configured" ]]; then
					sqlite3 $ZLLDB "replace into config2 (key, value) values('wifi', 'configured')" &> /dev/null
				fi

				putWifiUpdated '{"status":"ap-connected"}'
				return
			fi
		done

		putWifiUpdated '{"status":"ap-failed"}'
		TIMEOUT=5
		return
	fi # fi accesspoint

	### client mode
	if [[ "$wifiType" = "client" ]]; then
		[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}enter configure client section"

		if [ $ERROR_COUNT -gt $MAX_ERRORS ]; then
	        #restoreAP
	        [[ $LOG_ERROR ]] && echo "${LOG_ERROR}failed to connect wifi client, enable accesspoint as backup"
	        ERROR_COUNT=0
			RESTORE_AP=true
		    sqlite3 $ZLLDB "replace into config2 (key, value) values('wifitype', 'accesspoint-backup')" &> /dev/null
			sqlite3 $ZLLDB "replace into config2 (key, value) values('wifi', 'configured')" &> /dev/null
			putWifiUpdated '{"status":"client-failed"}'
	        return
	    fi

		for i in {4..5}; do
			param=${params[$i]}
			value=$(sqlite3 $ZLLDB "select * from config2 where key=\"${param}\"")
			if [ $? -ne 0 ]; then
				ERROR_COUNT=$((ERROR_COUNT + 1))
				return
			fi

			value=$(echo $value | cut -d'|' -f2)

			# basic check for non empty; not set
			if [[ ! -z "$value" && "$value" != "Not set" ]]; then
				values[$i]=$(echo $value | cut -d'|' -f2)
			fi
		done

		# all parameters found and valid?
		if [ -z "${values[4]}" ]; then
			[[ $LOG_ERROR ]] && echo "${LOG_ERROR}missing parameter wificlientname"
			ERROR_COUNT=$((ERROR_COUNT + 1))
			return
		fi
		if [ -z "${values[5]}" ]; then
			[[ $LOG_ERROR ]] && echo "${LOG_ERROR}missing parameter wificlientpw"
			ERROR_COUNT=$((ERROR_COUNT + 1))
			return
		fi

		which wpa_supplicant &> /dev/null
		if [ $? -ne 0 ]; then
			[[ $LOG_CRIT ]] && echo "${LOG_CRIT}wpa_supplicant not installed"
			ERROR_COUNT=$((ERROR_COUNT + 2)) # crit error
			return
		fi

		WIFI_NAME=${values[4]}
		WIFI_PW=${values[5]}

		# TODO use wpa_passphrase

		if [ -f "$WPA_CONF" ]; then
			# check from config
			OLD_WIFI_NAME=$(cat $WPA_CONF | grep "ssid" | cut -d'=' -f2 | cut -d\" -f2)
			OLD_WIFI_PW=$(cat $WPA_CONF | grep "psk" | cut -d'=' -f2 | cut -d\" -f2)
		fi

		# reconfigure if parameter changed or services aren't started
		if [[ -n $IP_ADDR ]] && [[ "$WIFI_NAME" == "$OLD_WIFI_NAME" ]] && [[ "$WIFI_PW" == "$OLD_WIFI_PW" ]]; then
			[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}config is up to date"
			pidof wpa_supplicant > /dev/null
			if [ $? -eq 0 ]; then
				[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}wpa_supplicant is running"

				iw $WIFI_DEV info 2> /dev/null | grep "$WIFI_NAME" > /dev/null
				if [ $? -eq 0 ]; then
					[[ $LOG_INFO ]] && echo "${LOG_INFO}connected to $WIFI_NAME"
					ERROR_COUNT=0
					putWifiUpdated '{"status":"client-alive"}'
					return
				fi
		    else
			    [[ $LOG_WARN ]] && echo "${LOG_WARN}wpa_supplicant is not running - reconfigure network"
			fi
		else
			[[ $LOG_INFO ]] && echo "${LOG_INFO}wifi config changed"
		fi # reconfigure if parameter changed

		## stop network
		[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}stopping network"

		systemctl stop networking
		systemctl stop dhcpcd
		systemctl stop hostapd
		systemctl stop dnsmasq

		# remove all ip addresses
		ip addr flush dev $WIFI_DEV

		systemctl stop wpa_supplicant
		pkill wpa_supplicant

		[[ $LOG_INFO ]] && echo "${LOG_INFO}configuring WIFI"

		## create wpa_supplicant conf
		echo "# generated by deCONZ
ctrl_interface=DIR=/var/run/wpa_supplicant GROUP=netdev
update_config=1

network={
ssid=\"${WIFI_NAME}\"
psk=\"${WIFI_PW}\"
}" > $WPA_CONF


if [ "$OS_ID" = "raspbian" ] && [ "$OS_VERSION" = "jessie" ]; then

		echo "# generated by deCONZ
# interfaces(5) file used by ifup(8) and ifdown(8)

# Please note that this file is written to be used with dhcpcd
# For static IP, consult /etc/dhcpcd.conf and 'man dhcpcd.conf'

# Include files from /etc/network/interfaces.d:
source-directory /etc/network/interfaces.d

auto lo
iface lo inet loopback

iface eth0 inet manual

allow-hotplug $WIFI_DEV
iface $WIFI_DEV inet manual
    wpa-conf /etc/wpa_supplicant/wpa_supplicant.conf

allow-hotplug wlan1
iface wlan1 inet manual
    wpa-conf /etc/wpa_supplicant/wpa_supplicant.conf" > $INTERFACES
fi


# stretch uses dhcpcd hooks for wpa_supplicant
if [ "$OS_ID" = "raspbian" ] && [ "$OS_VERSION" = "stretch" ]; then
			echo "# generated by deCONZ
# interfaces(5) file used by ifup(8) and ifdown(8)

# Please note that this file is written to be used with dhcpcd
# For static IP, consult /etc/dhcpcd.conf and 'man dhcpcd.conf'

# Include files from /etc/network/interfaces.d:
source-directory /etc/network/interfaces.d" > $INTERFACES
fi

		## dynamic ip addresses
			echo "# created by deCONZ
# A sample configuration for dhcpcd.
# See dhcpcd.conf(5) for details.

# Allow users of this group to interact with dhcpcd via the control socket.
#controlgroup wheel

# Inform the DHCP server of our hostname for DDNS.
hostname

# Use the hardware address of the interface for the Client ID.
clientid
# or
# Use the same DUID + IAID as set in DHCPv6 for DHCPv4 ClientID as per RFC4361.
# Some non-RFC compliant DHCP servers do not reply with this set.
# In this case, comment out duid and enable clientid above.
#duid

# Persist interface configuration when dhcpcd exits.
persistent

# Rapid commit support.
# Safe to enable by default because it requires the equivalent option set
# on the server to actually work.
option rapid_commit

# A list of options to request from the DHCP server.
option domain_name_servers, domain_name, domain_search, host_name
option classless_static_routes
# Most distributions have NTP support.
option ntp_servers
# Respect the network MTU. This is applied to DHCP routes.
option interface_mtu

# A ServerID is required by RFC2131.
require dhcp_server_identifier

# Generate Stable Private IPv6 Addresses instead of hardware based ones
slaac private
" > $DHCPCD_CONF

		## restart network
		[[ $LOG_INFO ]] && echo "${LOG_INFO}starting wifi client mode"

		putWifiUpdated '{"status":"client-connecting"}'

		systemctl daemon-reload
		systemctl restart dhcpcd
		systemctl restart networking

		pidof wpa_supplicant > /dev/null
		if [[ $? -ne 0 ]]; then
			# start wpa_supplicant manually, systemd doesn't seem to work
			wpa_supplicant -u -s -i $WIFI_DEV -c $WPA_CONF -B
		fi

		# reload config if already running
		#killall -HUP wpa_supplicant
		sleep 5
		local ok=0

		for i in {1..10}; do

			# connected to AP?
			wpa_cli -i $WIFI_DEV status | grep 'wpa_state' > /dev/null

			local mng=$(iw $WIFI_DEV info 2> /dev/null | grep "type managed")
			local ssid=$(iw $WIFI_DEV info 2> /dev/null | grep "ssid $WIFI_NAME")
			if [[ -n $mng ]] && [[ -n $ssid ]]; then
				# have ip address?
				local ip=$(ip -4 a show dev $WIFI_DEV up scope global)
				[[ -n $ip ]] && ok=1
			fi

			[[ $ok -eq 1 ]] && break

			putWifiUpdated '{"status":"client-connecting"}'

			sleep 2
		done

		if [[ $ok -eq 0 ]]; then
			ERROR_COUNT=$((ERROR_COUNT + 1))
			[[ $LOG_ERROR ]] && echo "${LOG_ERROR}could not start wifi client mode (error count $ERROR_COUNT)"
			TIMEOUT=5 # try again
			putWifiUpdated '{"status":"client-retry"}'
			return
		fi

		[[ $LOG_INFO ]] && echo "${LOG_INFO}client connection to wifi established"
		checkIpAddr

		if [[ "$wifiConfig" != "configured" ]]; then
			sqlite3 $ZLLDB "replace into config2 (key, value) values('wifi', 'configured')" &> /dev/null
		fi

		putWifiUpdated '{"status":"client-connected"}'

	fi # fi client mode

}

# Scan available wifi and push to REST-API
scanAvailableWifi() {

	[[ -z $WIFI_CONFIG ]] && return
	[[ "$WIFI_CONFIG" == "not-available" ]] && return
	[[ "$WIFI_CONFIG" == "deactivated" ]] && return


	[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}scan available wifi"
	local ssid=""
	local sig=""
	local ch=0
	local i=0

	# scan is processed in a subshell and returns a python array in NETW
	NETW=$(
	iw "$WIFI_DEV" scan 2> /dev/null |
	while IFS= read -r line
	do
		if [[ $line =~ BSS.*\(on.* ]]; then
			# next record
			ssid=""
			sig=""
			ch=""
			security=""
		fi

		if [[ $line = *"SSID:"* ]]; then
			ssid=`echo "$line" | cut -d':' -f2 | xargs`
		fi

		if [[ -z $signal ]] && [[ $line = *"signal:"* ]]; then
			sig=`echo "$line" | cut -d':' -f2 | xargs | cut -d ' ' -f1`
		fi

		if [[ -z $ch ]] && [[ $line = *"primary channel:"* ]]; then
			ch=`echo "$line" | cut -d':' -f2 | xargs`
		fi

		if [[ -z $ch ]] && [[ $line = *"DS Parameter set: channel"* ]]; then
			ch=`echo "$line" | cut -d':' -f2 | xargs | cut -d' ' -f2`
		fi

		if [[ -z $security ]] && [[ $line = *"RSN:"* ]]; then
			security="wpa2"
		fi

		if [[ -n $ssid ]] && [[ -n $sig ]] && [[ -n $ch ]] && [[ -n $security ]]
		then
			if [[ $i -gt 0 ]]; then
				echo -n ", "
			fi
			echo -n "{'ssid': '$ssid', 'channel': $ch, 'rssi': $sig }"
			ssid=""
			ch=""
			sig=""
			security=""
			i=1
		fi
	done)

	# discard empty result (might be busy)
	[[ -z $NETW ]] && return

	# use python to convert to JSON (incl. unicode stuff)
	local json=$(python -c "import json; print(json.dumps([$NETW]))")
	# -w '%{http_code}'
	curl --noproxy '*' -s -o /dev/null -d "$json" -X PUT http://127.0.0.1:${DECONZ_PORT}/api/$OWN_PID/config/wifi/scanresult
}

# get ip addr of gateway:
function checkIpAddr {
	[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}check ip address"

	local ip=$IP_ADDR

	IP_ADDR=$(ip -o -4 a show dev $WIFI_DEV up scope global | tr -s '[:blank:]' | cut -d' ' -f4 | cut -d'/' -f1)
	if [[ -n $IP_ADDR ]] && [[ $IP_ADDR =~ ^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$ ]]; then
		[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}got ip address $IP_ADDR"
		#ERROR_COUNT=0
	else
		[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}no ip address for $WIFI_DEV"
		IP_ADDR="" # unknown or error
	fi

	if [[ $ip != $IP_ADDR ]]; then
		# update database
		sqlite3 $ZLLDB "replace into config2 (key, value) values('wifiip', '$IP_ADDR')" &> /dev/null
		putWifiUpdated "{\"status\":\"got-ip\",\"ipv4\": \"$IP_ADDR\"}"
	fi
}

# break loop on SIGUSR1
trap 'TIMEOUT=0' SIGUSR1

while [ 1 ]
do
	if [[ -z "$ZLLDB" ]] || [[ ! -f "$ZLLDB" ]]; then
		init
	fi

	while [[ $TIMEOUT -gt 0 ]]
	do
		sleep 1
		TIMEOUT=$((TIMEOUT - 1))
	done

	TIMEOUT=30

	[[ -z "$ZLLDB" ]] && continue
	[[ ! -f "$ZLLDB" ]] && continue
	[[ -z "$WIFI_DEV" ]] && continue

    scanAvailableWifi

	checkIpAddr
	checkWifiConfig
done
